@@ -1092,9 +1092,14 @@ int MIMEH_read_headers( FFGET_FILE *f )
 
 	int is_RFC822_headers=0;	// 20040208-1335:PLD: Added to give an indication if the headers are RFC822 like; used in conjunction with the header_longsearch for pulling apart qmail bouncebacks.
 
-	// Lets start the ugly big fat DO loop here so that we can, if needed
-	//		search until we find headers which are actually valid.  Personally
-	//		I hate this - but people want it.
+	/**
+			Lets start the ugly big fat DO loop here so that we can, if needed
+			search until we find headers which are actually valid.  Personally
+			I hate this - but people want it in order to detect malformed 
+			(deliberate or otherwise) emails.  It'd be nice if for once in the
+			software world someone actually enforced standards rather than trying
+			to be overly intelligent about interpreting data.  
+	**/
 
 	if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_read_headers:DEBUG: File position = %ld [0x%0X]"
 			,FL
@@ -1979,6 +1984,7 @@ int MIMEH_parse_contenttype( char *header_name, char *header_value, struct MIMEH
 		else if (strstr(q,"multipart/alternative")) hinfo->content_type = _CTYPE_MULTIPART_ALTERNATIVE;
 		else if (strstr(q,"multipart/report")) hinfo->content_type = _CTYPE_MULTIPART_REPORT;
 		else if (strstr(q,"multipart/")) hinfo->content_type = _CTYPE_MULTIPART;
+		else if (strstr(q,"text/calendar")) hinfo->content_type = _CTYPE_TEXT_CALENDAR;
 		else if (strstr(q,"text/plain")) hinfo->content_type = _CTYPE_TEXT_PLAIN;
 		else if (strstr(q,"text/html")) hinfo->content_type = _CTYPE_TEXT_HTML;
 		else if (strstr(q,"text/")) hinfo->content_type = _CTYPE_TEXT;
