@@ -16,20 +16,21 @@ Description:   Altermime is a program/object which will allow arbitary alteratio
 #include "logger.h"
 
 /* Global DEFINES */
-char ALTERMIMEAPP_VERSION[]="alterMIME v0.3.7 (29-March-2006) by Paul L Daniels - http://www.pldaniels.com/altermime\n";
+char ALTERMIMEAPP_VERSION[]="alterMIME v0.3.8 (June-09-2007) by Paul L Daniels - http://www.pldaniels.com/altermime\n";
 char ALTERMIMEAPP_USAGE[]="altermime --input=<input mime pack>   ( --input=- for stdin )\n"
 "	[--disclaimer=<disclaimer file>]\n"
 "	[--disclaimer-html=<HTML disclaimer file>]\n"
+"	[--disclaimer-b64=<BASE64 encoded dislcaimer>]\n"
 "	[--htmltoo]\n"
 #ifdef ALTERMIME_PRETEXT
 "	[--pretext=<pretext file>]\n"
 "	[--pretext-html=<pretext HTML file>]\n"
 #endif
+"  [--force-into-b64]\n"
 "	[--force-for-bad-html]\n"
 "	[--multipart-insert]\n"
-"	[--remove=<remove file name>] (if filename contains a /, matches on mime-type )\n"
+"	[--remove=<remove file name (regex)>] (if filename contains a /, matches on mime-type )\n"
 "	[--removeall]\n"
-"	[--remove-by-regex=<filename regex>] (if the regex contains a /, matches on mime-type )\n"
 "	[--replace=<filename to replace> --with=<replace with>]\n"
 "	[--xheader=\"...\"\n"
 "	[--alter-header=\"...\" --alter-with=\"...\" --alter-mode=<prefix|suffix|replace>]\n"
@@ -45,12 +46,17 @@ char ALTERMIMEAPP_USAGE[]="altermime --input=<input mime pack>   ( --input=- for
 "\t--input=, Sets the mailpack file to be the filename supplied,\n"
 "\t\tif the filename is a single '-' (hyphen) then the mailpack\n"
 "\t\tis sourced via stdin and outputted via stdout.\n"
+"\n"
 "\t--disclaimer=, Set the plaintext disclaimer source file.\n"
 "\t--disclaimer-html=, Set the HTML disclaimer source file.\n"
+"\t--disclaimer-b64=, Set the BASE64 encoded disclaimer source file (implies --force-into-b64).\n"
+"\n"
 "\t--htmltoo, Sets alterMIME to insert the plaintext disclaimer into\n"
+"\t--force-into-b64, Sets alterMIME to insert disclaimers into BASE64 encoded text segments\n"
 "\t--force-for-bad-html, Force adding of the HTML disclaimer even when HTML is not correctly formatted\n"
 "\t\tthe HTML portion of the email body ( if there is no explicitly\n"
 "\t\tdefined HTML dislcaimer, see --disclaimer-html )\n"
+"\n"
 "\t--remove=, Remove any attachments which match the filename supplied,\n"
 "\t\tif the filename text contains a forward-slash '/', then the\n"
 "\t\tmatching will occur based on content-type headers rather than\n"
@@ -82,6 +88,7 @@ struct ALTERMIMEAPP_globals {
 	char *input_file;
 	char *disclaimer_file;
 	char *disclaimer_html_file;
+	char *disclaimer_b64_file;
 	char *pretext_file;
 	char *pretext_html_file;
 	int   pretext_insert;
@@ -126,6 +133,10 @@ int ALTERMIMEAPP_parse_args( struct ALTERMIMEAPP_globals *glb, int argc, char **
 			{
 				AM_set_HTMLtoo(1);
 			}
+			else if (strncmp(p,"force-into-b64",sizeof("force-into-b64"))==0)
+			{
+				AM_set_force_into_b64(1);
+			}
 			else if (strncmp(p,"force-for-bad-html",sizeof("force-for-bad-html"))==0)
 			{
 				AM_set_force_for_bad_html(1);
@@ -159,6 +170,11 @@ int ALTERMIMEAPP_parse_args( struct ALTERMIMEAPP_globals *glb, int argc, char **
 				AM_set_disclaimer_HTML( glb->disclaimer_html_file, AM_DISCLAIMER_TYPE_FILENAME );
 				AM_set_HTMLtoo(1);
 			}
+			else if (strncmp(p,"disclaimer-b64=",strlen("disclaimer-b64="))==0) {
+				glb->disclaimer_b64_file = p +strlen("disclaimer-b64=");
+				AM_set_disclaimer_b64( glb->disclaimer_b64_file, AM_DISCLAIMER_TYPE_FILENAME );
+				AM_set_force_into_b64(1);
+			}
 			else if (strncmp(p,"remove=",7)==0)
 			{
 				glb->remove_filename = p +strlen("remove=");
@@ -255,6 +271,7 @@ int ALTERMIMEAPP_init( struct ALTERMIMEAPP_globals *glb )
 	glb->input_file 				= NULL;
 	glb->disclaimer_file 		= NULL;
 	glb->disclaimer_html_file 	= NULL;
+	glb->disclaimer_b64_file	= NULL;
 
 #ifdef ALTERMIME_PRETEXT
 	glb->pretext_file = NULL;
