@@ -8,7 +8,7 @@ Description:   ffget is a small module which will be used to (we hope) speed up
 routine by line-buffering up first.
 
 12/11/2002: Corrected input buffer termination with fgets where the last line
-	of the input file does not terminate with a \n or \r.
+of the input file does not terminate with a \n or \r.
 
 27/09/2001:  Added SGI specific compile time changes from char -> short
 contributed by Don Lafontaine <lafont02@cn.ca>
@@ -23,11 +23,15 @@ contributed by Don Lafontaine <lafont02@cn.ca>
 #include "logger.h"
 #include "ffget.h"
 
-#define FFGET_VERSION	"1.0.0.4"
-#define FFGET_LASTUPDATED "29/03/2006"
+#ifndef FL
+#define FL __FILE__,__LINE__
+#endif
 
-#define FFGET_DNORMAL   ((FFGET_debug >= _FFGET_DEBUG_NORMAL  ))
-#define FFGET_DPEDANTIC ((FFGET_debug >= _FFGET_DEBUG_PEDANTIC))
+#define FFGET_VERSION	"1.0.0.7"
+#define FFGET_LASTUPDATED "200707091500"
+
+#define FFGET_DNORMAL   ((FFGET_debug >= FFGET_DEBUG_NORMAL  ))
+#define FFGET_DPEDANTIC ((FFGET_debug >= FFGET_DEBUG_PEDANTIC))
 
 /* GLOBALS */
 int ffget_linesize=0;
@@ -35,12 +39,13 @@ int FFGET_doubleCR = 0;
 int FFGET_SDL_MODE = 0;  // Single Char Delimeter
 
 int FFGET_SDL_WATCH = 0;	// Set if we want to watch for double-CR exploits
+int FFGET_ALLOW_NUL = 0;	// Dont Convert \0's to spaces.
 
 int FFGET_debug = 0;
 
-static char SDL_MODE_DELIMITS[]="\n\r";
-static char NORM_MODE_DELIMITS[]="\n";
-static char *DELIMITERS=SDL_MODE_DELIMITS;
+char SDL_MODE_DELIMITS[]="\n\r";
+char NORM_MODE_DELIMITS[]="\n";
+char *DELIMITERS=SDL_MODE_DELIMITS;
 
 /*------------------------------------------------------------------------
 Procedure:     FFGET_set_watch_SDL ID:1
@@ -58,6 +63,31 @@ int FFGET_set_watch_SDL( int level )
 	return FFGET_SDL_WATCH;
 }
 
+/*-----------------------------------------------------------------\
+  Function Name	: FFGET_set_allow_nul
+  Returns Type	: int
+  ----Parameter List
+  1. int level , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+This tells the FFGET_raw() if it needs to remove \0's or not
+in the data it reads.  This was added so as to ensure that \0
+sequences were preserved in form-data from WWW pages.
+
+--------------------------------------------------------------------
+Changes:
+Added 2004-Aug-09 by PLD.
+
+\------------------------------------------------------------------*/
+int FFGET_set_allow_nul(int level )
+{
+	FFGET_ALLOW_NUL = level;
+
+	return FFGET_ALLOW_NUL;
+}
 
 /*------------------------------------------------------------------------
 Procedure:     FFGET_set_debug ID:1
@@ -102,47 +132,55 @@ int FFGET_getnewblock( FFGET_FILE *f )
 	} else {
 		long block_pos;
 
-		block_pos = ftell(f->f);
-		bs = fread( f->buffer, 1, _FFGET_BUFFER_MAX -2, f->f );
+		block_pos = ftell(f->f); /** Get our current read position so we can use it in FFGET_ftell if required **/
 
-		if (bs < (_FFGET_BUFFER_MAX -2)) {
-			if (feof(f->f)) {
+		bs = fread( f->buffer, 1, FFGET_BUFFER_MAX -FFGET_BUFFER_PADDING, f->f );
+
+		if (bs < (FFGET_BUFFER_MAX -FFGET_BUFFER_PADDING))
+		{
+			if (feof(f->f))
+			{
 				f->FILEEND = 1;
-
-			} else {
-
-				fprintf(stderr,"%s:%d: WARNING - File read failed with error:%s",__FILE__,__LINE__,strerror(errno));
+			}
+			else
+			{
+				LOGGER_log("%s:%d:FFGET_getnewblock:ERROR: File read failed with error:%s", FL, strerror(errno));
 				return 0;
 			}
 		}
 
-		if (bs > 0) {
+		if (bs > 0)
+		{
 
 			// If we read in some data, then adjust the buffer to deal with it
-		//
-		// First we set the start point back to the start of the buffer,
-		// then we set the end point to be the start +datasize we read, -1
-		// then we adjust the total bytes read (for the sake of record keeping
-		// though it has no /real/ purpose)
-		//
+			//
+			// First we set the start point back to the start of the buffer,
+			// then we set the end point to be the start +datasize we read, -1
+			// then we adjust the total bytes read (for the sake of record keeping
+			// though it has no /real/ purpose)
+			//
 
-			f->last_block_read_from = block_pos;
+			f->buffer[bs] = '\0';	//20040208-1703:PLD:JS
+			f->last_block_read_from = block_pos; // 200607150941:PLD
 			f->startpoint = f->buffer;
 			f->endpoint = f->startpoint +bs -1;
 			f->bytes += bs;
 
 			// Check the buffer for poisioning \0's
-		//  As these routines are being used for 7-bit valid text data,
-		// we have to filter out any nasty \0's.
+			//  As these routines are being used for 7-bit valid text data,
+			// we have to filter out any nasty \0's.
 
-			p = f->startpoint;
-			for (i = 0; i < bs; i++) {
-				if (*p == '\0') *p = ' ';
-				p++;
+			if (FFGET_ALLOW_NUL == 0)
+			{
+				p = f->startpoint;
+				for (i = 0; i < bs; i++) {
+					if (*p == '\0') *p = ' ';
+					p++;
+				}
+				*p = '\0';
 			}
-			*p = '\0';
 
-			if (FFGET_DPEDANTIC) LOGGER_log("Size: %ld bytes\n",f->bytes);
+			if (FFGET_DPEDANTIC) LOGGER_log("%s:%d:FFGET_getnewblock:DEBUG-PEDANTIC: Size: %ld bytes\n", FL, f->bytes);
 
 		}
 
@@ -167,7 +205,7 @@ Errors:
 ------------------------------------------------------------------------*/
 int FFGET_presetbuffer( FFGET_FILE *f, char *buffer, int size )
 {
-	if (size > _FFGET_BUFFER_MAX) size = _FFGET_BUFFER_MAX;
+	if (size > FFGET_BUFFER_MAX) size = FFGET_BUFFER_MAX;
 
 	memcpy(f->buffer,buffer,size);
 	f->startpoint = buffer;
@@ -186,15 +224,19 @@ Errors:
 ------------------------------------------------------------------------*/
 int FFGET_setstream( FFGET_FILE *f, FILE *fi )
 {
+
+	memset(f,0,sizeof(FFGET_FILE)); // be pedantic - clear the struct
+
 	f->f = fi;
 	f->bytes = 0;
 	f->linecount = 0;
 	f->endpoint = f->buffer;
 	f->startpoint = f->endpoint +1;
+	f->buffer_end = f->buffer +FFGET_BUFFER_MAX +FFGET_BUFFER_PADDING;
 	f->trueblank = 0;
 	f->ungetcset = 0;
 	f->lastchar = '\0';
-	memset(f->buffer,'\0',_FFGET_BUFFER_MAX +1);
+	memset(f->buffer,'\0',FFGET_BUFFER_MAX +FFGET_BUFFER_PADDING);
 	f->c = '\0';
 	f->FFEOF = 0;
 	f->FILEEND = 0;
@@ -229,50 +271,63 @@ int FFGET_feof( FFGET_FILE *f )
 }
 
 
+
 /*-----------------------------------------------------------------\
- Function Name	: FFGET_seek
- Returns Type	: int
- 	----Parameter List
-	1. FFGET_FILE *f, 
-	2.  size_t offset , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: FFGET_seek
+  Returns Type	: int
+  ----Parameter List
+  1. FFGET_FILE *f, 
+  2.  size_t offset , 
+  ------------------
+  Exit Codes	: 
+  -1 = error, check logs for reason of failure.
+
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+Seeks to 'offset' bytes from the first byte of the file.
+Reloads the buffer block.
+
 --------------------------------------------------------------------
- Comments:
-		Seeks to 'offset' bytes from the first byte of the file.
-		Reloads the buffer block.
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int FFGET_seek( FFGET_FILE *f, long offset, int whence )
 {
 	int result = 0;
-	fseek(f->f, offset, whence);
+
+	/** Move to the new block location **/
+	result = fseek(f->f, offset, whence);
+	if (result == -1) {
+		LOGGER_log("%s:%d:FFGET_seek:ERROR: While attempting to seek to offset %ld from %d - [%s]", FL, offset, whence, strerror(errno));
+		return -1;
+	}
+
+	/** Read a whole new block **/
 	result = FFGET_getnewblock(f);
+
 	return result;
 }
 
 
-/*-----------------------------------------------------------------\
- Function Name	: FFGET_tell
- Returns Type	: size_t
- 	----Parameter List
-	1. FFGET_FILE *f , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
---------------------------------------------------------------------
- Comments:
-		Returns the position in the file that the current "file cursor"
-		is pointing to.
 
- 
+/*-----------------------------------------------------------------\
+  Function Name	: FFGET_tell
+  Returns Type	: size_t
+  ----Parameter List
+  1. FFGET_FILE *f , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+Returns the position in the file that the current "file cursor"
+is pointing to.
+
+
 --------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 long FFGET_ftell( FFGET_FILE *f )
 {
@@ -284,6 +339,9 @@ long FFGET_ftell( FFGET_FILE *f )
 
 }
 
+
+
+
 /*------------------------------------------------------------------------
 Procedure:     FFGET_ungetc ID:1
 Purpose:       Pushes back into the buffer (effectively) a single character
@@ -334,7 +392,7 @@ char FFGET_fgetc( FFGET_FILE *f )
 		f->startpoint++;
 	}
 	else
-	    {
+	{
 		c = EOF;
 	}
 
@@ -360,17 +418,14 @@ Errors:
 char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 {
 	char *line = linein;
-	char *crlfpos;
-	int charstoCRLF;
-	int chardiff;
-	int result;
+	char *crlfpos = NULL;
+	int charstoCRLF = 0;
+	int chardiff = 0;
+	int result = 0;
 	int max_size = maxsize;
-
-
-	// We /ONLY/ return EOF -here-, because, even if during the process of
-	// reading through the bufffer we reach the end, we do still have data
-	// which we can send back to the caller.  I'm surprised that more people
-	// didn't pick up on this, it's a glaring flaw.
+	int endpoint_tainted = 0;
+	int extra_char_kept=0;
+	int c, nextchar;
 
 	f->trueblank = 0;
 
@@ -386,16 +441,16 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 	else DELIMITERS = NORM_MODE_DELIMITS;
 
 
-//	fprintf(stderr,"FFGET_called, SDLMODE = %d, Offset = %d, maxsize = %d, DATA left = %d, first char is '%02X'\n", FFGET_SDL_MODE, (f->startpoint -f->buffer), max_size, (f->endpoint -f->startpoint)+1, (int)(*f->startpoint));
+	//	fprintf(stderr,"FFGET_called, SDLMODE = %d, Offset = %d, maxsize = %d, DATA left = %d, first char is '%02X'\n", FFGET_SDL_MODE, (f->startpoint -f->buffer), max_size, (f->endpoint -f->startpoint)+1, (int)(*f->startpoint));
 
 	max_size = maxsize = maxsize -2;
 
-//	memset(line, 0, max_size+1);
+	//	memset(line, 0, max_size+1);
 
 	// If we dont have enough data in the buffer to fill up the fgets request
 	// we'll have to do a two step fill
 
-//fprintf(stderr,"DATA Reminianing : %d\n", (int)(f->endpoint -f->startpoint)+1);
+	//fprintf(stderr,"DATA Reminianing : %d\n", (int)(f->endpoint -f->startpoint)+1);
 
 	if ((f->startpoint > f->endpoint))
 	{
@@ -417,13 +472,16 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 		crlfpos = strpbrk( f->startpoint, DELIMITERS);
 		if (crlfpos)
 		{
+				extra_char_kept = 0;
+				endpoint_tainted = 0;
+				nextchar = -1;
 			// if our next char is a CR or LF, then pick it up and
 			// return it with the line.  NOTE - this is to deal with
 			// CRLF pairings which are common on DOS files.  In fact,
 			// this is a case of where UNIX is actually -wrong-.  It
 			// should have also used CRLF pairing to mark line ends, but
 			// someone obviously (and understandably, to save space)
-		   	// thought they'd leave make LF imply a CR as well.
+			// thought they'd leave make LF imply a CR as well.
 			//   Well done... another bugger up in life.
 
 
@@ -443,12 +501,11 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 
 				// We have an EOL character, get 1 more from the stream to test the next character
 
-				int c;
 
-				c = fgetc(f->f);
+				nextchar = c = fgetc(f->f);
 				if (c==EOF)
 				{
-//					fprintf(stderr,"EOF hit due to fgetc()\n");
+					//					fprintf(stderr,"EOF hit due to fgetc()\n");
 					f->FILEEND = 1;
 				}
 				else
@@ -459,9 +516,11 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 
 					if ((c > 0) && (c <= 255))
 					{
+						if (FFGET_DNORMAL) LOGGER_log("%s:%d:FFGET_fgets:DEBUG: Tainting endpoint +1 (%p -> %p,  hard buffer end = %p, file read bytes = %ld)", FL, f->endpoint, f->endpoint+1, f->buffer_end, f->bytes);
 						f->endpoint++;
 						*(f->endpoint) = c;
 						*(f->endpoint+1) = '\0';
+						endpoint_tainted = 1;
 					}
 
 				}
@@ -474,13 +533,14 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 			if ( ((crlfpos +1) <= f->endpoint))
 			{
 
-//				fprintf(stderr,"Found '%02X' [next is '%02X']\n",*crlfpos, *(crlfpos+1));
+				//				fprintf(stderr,"Found '%02X' [next is '%02X']\n",*crlfpos, *(crlfpos+1));
 
 				if ( *crlfpos == '\n' )
 				{
 					if ( *(crlfpos +1) == '\r' )
 					{
 						crlfpos++;
+						extra_char_kept = 1;
 					}
 				}
 
@@ -501,17 +561,18 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 						FFGET_doubleCR=1;
 						FFGET_SDL_MODE=1;
 						crlfpos++;
-					}
-					else
-					    if ( *(crlfpos +1) == '\n' )
-					{
+						extra_char_kept = 1;
+
+					} else if ( *(crlfpos +1) == '\n' ) {
 						// If we see a \n after our \r, then treat this as a single
 						// 	line delimeter if we are NOT in Single Delimeter mode
 
-						if (!FFGET_SDL_MODE) { crlfpos++; }
-					}
-					else
-					    {
+						if (!FFGET_SDL_MODE) {
+						  	crlfpos++;
+							extra_char_kept = 1;
+					  	}// 20040208-1706:PLD
+						//crlfpos++;// 20040208-1706:PLD	 // 20040306-0003:PLD - this line causes a CRCR test to fail; mailpack.virus.badtrans
+					} else {
 						// If we saw a \r, but then there was no other EOL type char (\r or \n)
 						//	then switch to SDL mode (Single delimeter).
 
@@ -537,10 +598,12 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 
 			if ((charstoCRLF >= 0)&&(charstoCRLF < max_size)) max_size = charstoCRLF;
 
+			if ((extra_char_kept == 0) && (nextchar != -1)) ungetc(nextchar,f->f);
+
 		} // If CRLF pos found.
 
 
-//		else crlfpos = (f->endpoint +1);
+		//		else crlfpos = (f->endpoint +1);
 
 
 
@@ -551,22 +614,21 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 
 		if (( f->endpoint -f->startpoint) >= max_size)
 		{
-			if (max_size < 0) fprintf(stderr,"ERROR - Max size < 0\n");
+			if (max_size < 0) LOGGER_log("%s:%d:FFGET_fgets:ERROR: Max size < 0\n", FL);
 			memcpy(line, f->startpoint, max_size +1);//+1
 			f->startpoint += (max_size +1); //+1
 			*(line +max_size +1) = '\0'; //+1
 			max_size = 0;
-		}
-		else
-		    {
+
+		} else {
 
 			// else, if the amount of data available is /LESS/ than what we can
-		// accept in the line buffer then copy what remains out to the line
-		// buffer and then tack on the results of a new read.
+			// accept in the line buffer then copy what remains out to the line
+			// buffer and then tack on the results of a new read.
 
 			chardiff = f->endpoint -f->startpoint;
 
-//			fprintf(stderr,"CHARDiff = %d, FFEOF = %d, FILEEND = %d\n",chardiff, f->FFEOF, f->FILEEND);
+			//			fprintf(stderr,"CHARDiff = %d, FFEOF = %d, FILEEND = %d\n",chardiff, f->FFEOF, f->FILEEND);
 
 			if (chardiff >= 0)
 			{
@@ -579,9 +641,15 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 			}
 
 			FFGET_getnewblock(f);
+			endpoint_tainted=0;
 
 		} // If there wasn't enough data to satisfy ends.
 
+		if (endpoint_tainted) {
+			FFGET_getnewblock(f);
+			endpoint_tainted = 0;
+		}
+
 	} // While we've got space to fill, and we've got data to read
 
 	line = linein;
@@ -600,6 +668,8 @@ char *FFGET_fgets( char *linein, int maxsize, FFGET_FILE *f )
 
 	f->linecount++;
 
+	//	LOGGER_log("%s:%d:LINE='%s'",FL,linein);
+
 	return linein;
 }
 
@@ -636,7 +706,7 @@ int FFGET_raw( FFGET_FILE *f, unsigned char *buffer, int max )
 		bytestogo = FFGET_getnewblock(f);
 	}
 	else
-	    {
+	{
 		bytestogo = f->endpoint -f->startpoint +1;
 	}
 
