@@ -39,11 +39,13 @@ struct AM_disclaimer_details {
 	int isfile;
 	int text_inserted;
 	int html_inserted;
+	int b64_inserted;
 
 	//
 
 	char *disclaimer_text_plain;
 	char *disclaimer_text_HTML;
+	char *disclaimer_text_b64;
 
 };
 
@@ -55,6 +57,7 @@ int AM_init( void );
 int AM_done( void );
 int AM_set_debug( int level );
 
+char *AM_set_disclaimer_b64( char *filename, int disclaimer_type );
 char *AM_set_disclaimer_plain( char *filename, int disclaimer_type );
 char *AM_set_disclaimer_HTML( char *filename, int disclaimer_type );
 
@@ -69,6 +72,7 @@ int AM_nullify_attachment( char *mpackname, char *attachmentname );
 int AM_set_verbose( int level );
 int AM_set_HTMLtoo( int level );
 int AM_set_force_for_bad_html( int level );
+int AM_set_force_into_b64( int level );
 int AM_set_multipart_insert( int level );
 int AM_set_nullifyall( int level );
 int AM_set_altersigned( int level );
@@ -78,4 +82,6 @@ int AM_attachment_replace( char *mpackname, char *attachmentname, char *new_atta
 int AM_insert_Xheader( char *fname, char *xheader);
 int AM_alter_header( char *filename, char *header, char *change, int change_mode );
 
+//int AM_force_into_b64( int level );
+
 #endif
